/*
** $Id: agnhlps.h $
** C Helper routines
** See Copyright Notice in agena.h
** initiated Alexander Walz, July 20, 2007
*/

#ifndef agnhlps_h
#define agnhlps_h

#include <stdio.h>
#include <sys/types.h>
#include <ctype.h>
#include <stdlib.h>       /* div_t */
#if defined(LUA_DOS) || defined(OPENSUSE)
#  include <stdint.h>     /* for int32_t */
#  include <sys/types.h>  /* for off_t & off64_t */
#endif

#include "agena.h"
#include "agncmpt.h"
#include "agnt64.h"
#include "cephes.h"
#include "sunpro.h"  /* EXTRACT_WORDS */
#include "prepdefs.h"

/* inline ... */

#if defined __GNUC__ && !defined __GNUC_STDC_INLINE__ && !defined __GNUC_GNU_INLINE__
#  define __GNUC_GNU_INLINE__ 1
#endif

#ifndef NOINLINE
#define INLINE inline
#else
#define INLINE
#endif

/* TYPEDEF 2.14.4 */

#ifndef off64_t
#  define off64_t long long int
#endif


#define SWAP(a,b,t) { (t) = (a); (a) = (b); (b) = (t); }

/* some constants */

#undef SQRT2
#define SQRT2        1.41421356237309504880168872421

#undef PI
#define PI           3.14159265358979323846264338328

#undef PI2
#define PI2          6.28318530717958647692528676656  /* Pi*2 */

#undef PIO2
#define PIO2         1.57079632679489661923132169164  /* Pi/2 */

#undef PIO4
#define PIO4         0.78539816339744830961566084582

#undef INVPI2
#define INVPI2       0.15915494309189533576888376337  /* 1/(Pi*2) */

#undef INVPIO4
#define INVPIO4      1.27323954473516268615107010698  /* 4/Pi, 2.14.6 */

#undef INVPISQO4
#define INVPISQO4    0.40528473456935108577551785284  /* 4/Pi^2, 2.14.6 */

#undef _E
#define _E           2.71828182845904523536028747135

#undef AGN_EPSILON
#define AGN_EPSILON  1.4901161193847656e-08  /* suited for doubles */

#undef AGN_HEPSILON  /* 2.12.1 */
#define AGN_HEPSILON 1.4901161193847656e-12

#undef RADIANS_PER_DEGREE
#define RADIANS_PER_DEGREE (PI/180.0)

#undef EULERGAMMA
#define EULERGAMMA   0.577215664901532860606512090082  /* Maple V Release 4: Digits := 30; evalf(gamma) */

#undef LN2
#define LN2	         0.693147180559945309417232121458  /* = ln(2) */

#undef INVLN2
#define INVLN2       1.44269504088896340735992468100  /* = 1/ln(2) */

#undef LN10
#define LN10	     2.30258509299404568401799145468   /* = ln(10) */

#undef M_SQRT1_2
#define M_SQRT1_2    0.707106781186547524400844362105  /* = 1/sqrt(2) */

#undef SQRTPI
#define SQRTPI       1.77245385090551602729816748334  /* = sqrt(Pi) */

#undef SQRTPI2
#define SQRTPI2      2.50662827463100050241576528481  /* = sqrt(2*Pi) */

#undef SQRT5
#define SQRT5        2.23606797749978969640917366873  /* = sqrt(5) */

#undef PHI
#define PHI          1.61803398874989484820458683437  /* = ((1.0 + sqrt(5.0))/2.0), Golden ratio, changed 2.9.8 */

#undef PHILN
#define PHILN        0.481211825059603447497758913427  /* = ln(math.Phi) */

#undef INVPHILN
#define INVPHILN     2.07808692123502753760132260611   /* = 1/ln(math.Phi) */

#define tools_b64    "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
#define tools_cb64   "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"

/* code taken from: @(#)s_nextafter.c 5.1 93/09/24
 *
 * ====================================================
 * Copyright (C) 1993 by Sun Microsystems, Inc. All rights reserved.
 *
 * Developed at SunPro, a Sun Microsystems, Inc. business.
 * Permission to use, copy, modify, and distribute this
 * software is freely granted, provided that this notice
 * is preserved.
 * ====================================================
 *
 * Get two 32 bit ints from a double.
 *
 * IEEE structure for doubles
 * taken from www.koders.com /..GNU/l/libc/libc/math/s_nextafter.c
 * because the original Sun implementation is Non-ANSI C and does not work with GCC.
 *
 * The macros unfortunately do not work within Lua/Agena library C functions, so we do not export them. */


#if BYTE_ORDER == BIG_ENDIAN
typedef union {
  double value;
  struct {
    uint32_t msw;
    uint32_t lsw;
  } parts;
} __ieee_double_shape_type;
#else
typedef union {
  double value;
  struct {
    uint32_t lsw;
    uint32_t msw;
  } parts;
} __ieee_double_shape_type;
#endif

#define __EXTRACT_WORDS(ix0,ix1,d) \
do { \
  __ieee_double_shape_type ew_u; \
  ew_u.value = (d); \
  (ix0) = ew_u.parts.msw; \
  (ix1) = ew_u.parts.lsw; \
} while (0)

#define __GET_HI(ix0,d) \
do { \
  __ieee_double_shape_type ew_u; \
  ew_u.value = (d); \
  (ix0) = ew_u.parts.msw; \
} while (0)

#define __GET_LO(ix1,d) \
do { \
  __ieee_double_shape_type ew_u; \
  ew_u.value = (d); \
  (ix1) = ew_u.parts.lsw; \
} while (0)

/* Set a double from two 32 bit ints.  */

#define __INSERT_WORDS(d,ix0,ix1) \
do { \
  __ieee_double_shape_type iw_u; \
  iw_u.parts.msw = (ix0); \
  iw_u.parts.lsw = (ix1); \
  (d) = iw_u.value; \
} while (0)

/* Set the more significant 32 bits of a double from an int.  */

#define __SET_HI(d,v) \
do { \
  __ieee_double_shape_type sh_u; \
  sh_u.value = (d); \
  sh_u.parts.msw = (v); \
  (d) = sh_u.value; \
} while (0)

/* Set the less significant 32 bits of a double from an int.  */

#define __SET_LO(d,v) \
do { \
  __ieee_double_shape_type sl_u; \
  sl_u.value = (d); \
  sl_u.parts.lsw = (v); \
  (d) = sl_u.value; \
} while (0)


/* misc functions */

/* used in packages `ads` */

LUALIB_API void tools_quicksort (off64_t v[], long int left, long int right);
LUALIB_API int tools_binsearch (off64_t *a, off64_t cnt, off64_t z);

/* used in packages `stats` */

LUALIB_API void tools_dquicksort (double v[], long int left, long int right);

LUALIB_API int pixel_qsort (double *pix_arr, int npix);

LUALIB_API void tools_dheapsort (double *a, long int l, long int r);

LUALIB_API void tools_dintrosort (double v[], long int left, long int right, size_t depth, size_t threshold);

LUALIB_API double tools_kth_smallest (double a[], long int n, long int k);

/* Dr. F.H.Toor's and my database C functions used in packages `base` and `phonetiqs` */

LUALIB_API int     my_open (const char *file);
LUALIB_API FILE    *my_fopen (const char *file);
LUALIB_API int     my_roopen (const char *file);
LUALIB_API FILE    *my_froopen (const char *file);
LUALIB_API int     my_create (const char *file);
LUALIB_API FILE    *my_fcreate (const char *file);
LUALIB_API int     my_close (int hnd);
LUALIB_API off64_t my_seek (int hnd, off64_t pos);
LUALIB_API off64_t my_lof (int hnd);
LUALIB_API void    my_read (int hnd, void *data, size_t size);
LUALIB_API int32_t my_readl (int hnd);
LUALIB_API char    my_readc (int hnd);
LUALIB_API void    my_write (int hnd, void *data, size_t size);
LUALIB_API void    my_writel (int hnd, int32_t data);
LUALIB_API void    my_writec (int hnd, char data);
LUALIB_API int     my_lock (int hnd, off64_t start, off64_t size);
LUALIB_API int     my_unlock (int hnd, off64_t start, off64_t size);
LUALIB_API void    my_move (int hnd, off64_t fpos, off64_t tpos, off64_t size);
LUALIB_API void    my_expand (int hnd, int mrc, int cnt, int count, int *error);
LUALIB_API off64_t my_fpos (int hnd);
LUALIB_API const char *my_ioerror (int en);

LUALIB_API size_t  sec_read (int hnd, void *data, size_t size);
LUALIB_API int32_t sec_readl (int hnd, ssize_t *success);
LUALIB_API uint32_t sec_readul (int hnd, ssize_t *success);

/* string functions */

LUALIB_API char     *charreplace (char *s, char from, char to, int flag);
LUALIB_API char     *concat (const char *s1, ...);
LUALIB_API int      glob (const char *pat, const char *str);
LUALIB_API char     *substr (const char *str, const size_t begin, const size_t end, int *error);
LUALIB_API char     *latin9_to_utf8 (const char *string);
LUALIB_API char     *utf8_to_latin9 (const char *string);
LUALIB_API size_t   utf8_to_latin9_len (const char *string, size_t *len);
LUALIB_API int      is_utf8 (const char *string, size_t *pos);
LUALIB_API size_t   size_utf8 (const char *str);
LUALIB_API int      tools_isnumericstring (const char *str);
LUALIB_API char     *strreplace (const char *original, const char *pattern, const char *replacement);
LUALIB_API void     tools_strrev (char *str);
LUALIB_API int      tools_stricmp (const char* p, const char* q);
LUALIB_API char     *tools_itoa (int64_t val, int base);
LUALIB_API INLINE ptrdiff_t tools_posrelat (ptrdiff_t pos, size_t len);
LUALIB_API INLINE size_t tools_optstrlen (size_t l, size_t *chunks);
LUALIB_API INLINE char *tools_stralloc (size_t len);
#define AGN_BLOCKSIZE   sizeof(unsigned long)  /* string "long" word-boundary */
LUALIB_API char     *tools_getline (FILE *fp, size_t *linesize, int *r);

/* Returns non-zero if x is aligned on a "long" boundary. */
#define tools_strisaligned(x) \
  (((long)(x) & (sizeof(long) - 1)) == 0)
#define tools_strisunaligned(x) \
  (((long)(x) & (sizeof(long) - 1)) != 0)
#define tools_stringsarealigned(x, y)   (tools_strisaligned(x) && tools_strisaligned(y))
#define tools_largeenough(l)  ((l) >= AGN_BLOCKSIZE)

/* Returns non-zero if x (a long int) contains a NULL byte. */
#if LONG_MAX == 2147483647L
#define tools_strisnull(x) (((x) - 0x01010101) & ~(x) & 0x80808080)
#else
#if LONG_MAX == 9223372036854775807L
#define tools_strisnull(x)     (((x) - 0x0101010101010101) & ~(x) & 0x8080808080808080)
#else
#error unusual long int size
#endif
#endif
#define tools_strisnonnull(x)  (!tools_strisnull(x))

LUALIB_API int tools_hasstrchr (const void *s, const void *p);
/* 4-byte-alignment computation does not work without knowing the sizes of the strings, so we define a macro: */
#define tools_streq(s, t) (strcmp((s), (t)) == 0)
LUALIB_API int tools_streqx (const char *s, ...);
LUALIB_API INLINE void *tools_memchr (const void *src, int c, size_t l);
LUALIB_API INLINE int tools_memcmp (const void *haystack, const void *needle, size_t l);
LUALIB_API INLINE const char *tools_lmemfind (const char *s1, size_t l1, const char *s2, size_t l2);

/* ISO 8859/1 Latin-1 alphabetic and upper and lower case bit vector tables.

   Taken from the entropy utility ENT written by John Walker, January 28th, 2008,
   Fourmilab, http://www.fourmilab.ch.

   This software is in the public domain. Permission to use, copy, modify, and distribute this software
   and its documentation for any purpose and without fee is hereby granted, without any conditions or
   restrictions. This software is provided as is without express or implied warranty. */

LUALIB_API unsigned char isoalpha[32], isoupper[32], isolower[32];

#define isISOspace(x)	((isascii((uchar (x))) && isspace((uchar (x)))) || ((x) == 0xA0))
#define isISOalpha(x)	((isoalpha[((uchar (x))) / 8] & (0x80 >> (((uchar (x))) % 8))) != 0)
#define isISOupper(x)	((isoupper[((uchar (x))) / 8] & (0x80 >> (((uchar (x))) % 8))) != 0)
#define isISOlower(x)	((isolower[((uchar (x))) / 8] & (0x80 >> (((uchar (x))) % 8))) != 0)
#define isISOprint(x)   ((((x) >= ' ') && ((x) <= '~')) || ((x) >= 0xA0))
#define toISOupper(x)   (isISOlower(x) ? (isascii((uchar (x))) ?  \
                            toupper(x) : ((((uchar (x)) != 0xDF) && \
                            ((uchar (x)) != 0xFF)) ? \
			                ((uchar (x)) - 0x20) : (x))) : (x))
#define toISOlower(x)   (isISOupper(x) ? (isascii((uchar (x))) ?  \
                            tolower(x) : ((uchar (x)) + 0x20)) \
			                : (x))

/* miscellaneous and OS functions */
LUALIB_API char *tools_getenv (const char *name);
LUALIB_API int tools_cwd (char *buffer);
/* get keystroke function, does not work in Haiku */
#if (defined(__unix__) && !defined(LUA_DOS)) || defined(__APPLE__) || defined(__OS2__)
LUALIB_API int kbhit (void);
#ifndef __OS2__  /* getch works in eCS */
LUALIB_API int getch (void);
#endif
#endif
LUALIB_API int tools_anykey (void);  /* 2.12.4 */

#define xfree(v) { free(v); v = NULL; }

/* Taken from `21st Century C`, 2nd edition, O'Reilly, p. 182 of German translation, vectorise.c source file,
   by Ben Klemens; modified; needs stdlib.h for free function */
#define Fn_freeall(type, fn, ...) {                                   \
  int i;                                                              \
  void *stopper_for_apply = (int[]){0};                               \
  type **list_for_apply = (type*[]){__VA_ARGS__, stopper_for_apply};  \
  for (i=0; list_for_apply[i] != stopper_for_apply; i++) {            \
    fn(list_for_apply[i]);                                            \
    list_for_apply[i] = NULL;                                         \
  }                                                                   \
}

#define xfreeall(...) Fn_freeall(void, free, __VA_ARGS__);


/* Endian functions */

LUALIB_API char     tools_endian (void);
LUALIB_API double   tools_tolittleendian (double x);
LUALIB_API double   tools_tobigendian (double x);

#if BYTE_ORDER != BIG_ENDIAN
#if __GNUC_PREREQ(4, 3)
#define tools_swapdouble   __builtin_bswap64
#else
#define tools_swapdouble(x)                 \
  (  (((x) & 0xff00000000000000ull) >> 56)  \
   | (((x) & 0x00ff000000000000ull) >> 40)  \
   | (((x) & 0x0000ff0000000000ull) >> 24)  \
   | (((x) & 0x000000ff00000000ull) >> 8)   \
   | (((x) & 0x00000000ff000000ull) << 8)   \
   | (((x) & 0x0000000000ff0000ull) << 24)  \
   | (((x) & 0x000000000000ff00ull) << 40)  \
   | (((x) & 0x00000000000000ffull) << 56))
#endif
#define tools_unswapdouble  tools_swapdouble
#else
LUALIB_API uint64_t tools_swapdouble (double n);
LUALIB_API double   tools_unswapdouble (uint64_t n);
#endif

LUALIB_API void     tools_swaplong (long *n);
LUALIB_API void     tools_swapint (int *n);
LUALIB_API int32_t  tools_swapint32 (int32_t d);
LUALIB_API void     tools_swapint32_t (int32_t *n);
LUALIB_API void     tools_swapuint32_t (uint32_t *n);
/* taken from: https://stackoverflow.com/questions/48737056/bit-cast-uint64-t-to-double-and-back-in-a-macro
   3.5 % faster than the original iterative casting functions; as fast as "*((uint64_t*)(void*)&x" or
   uint64_t u;
   double x = ...;
   memcpy(&u, &x, sizeof(u)); */
#define tools_doubletouint64(x) ((union { double d; uint64_t u; }) { x } .u)
#define tools_uint64todouble(x) ((union { uint64_t u; double d; }) { x } .d)
/* 2.16.6 fix */
#if BYTE_ORDER == BIG_ENDIAN
LUALIB_API double   tools_uint64todoubleandswap (uint64_t a);
LUALIB_API uint64_t tools_doubletouint64andswap (double a);
#endif
LUALIB_API uint64_t tools_twoint32touint64 (int32_t d, int32_t e);
LUALIB_API int32_t  tools_uint64toint32 (uint64_t d, char k);
LUALIB_API double   tools_sint2double (int32_t a);

/* IEEE simulators */

LUALIB_API int      tools_isfinite (const double x);
LUALIB_API int      tools_isnan (const double x);
LUALIB_API double   tools_nan (void);

/* numerical functions */

LUALIB_API double   tools_intpow (double x, int n);
LUALIB_API float    tools_pow2fast (float x);
LUALIB_API int      tools_approx (double x, double y, double eps);
LUALIB_API double   tools_csgn (double a, double b);
LUALIB_API double   tools_roundf (double x, int d, int t);
LUALIB_API double   tools_fdim (double x, double y);
LUALIB_API int      tools_signbit (double x);
LUALIB_API double   tools_fma (double a, double b, double c);
LUALIB_API INLINE double   tools_mhypot (double a, double b);
LUALIB_API INLINE double   sun_hypot (double x, double y);
LUALIB_API double   sun_hypot2 (double y);
LUALIB_API double   sun_hypot3 (double y);
LUALIB_API int      tools_uintlog2 (uint32_t v);
LUALIB_API float    tools_lbfast (float x);
LUALIB_API double   sun_nextafter (double x, double y);
LUALIB_API int      sun_frexp_exp (double x);
LUALIB_API double   sun_frexp_man (double x);
LUALIB_API double   sun_xfrexp (double x, int *eptr, int *isneg);
LUALIB_API INLINE int32_t sun_exponent (double x);
LUALIB_API INLINE int sun_irregular (double x);
LUALIB_API INLINE int tools_isregint (double x);
LUALIB_API INLINE int tools_isregnonnegint (double x);
LUALIB_API int      sun_subnormal (double x);
LUALIB_API double   sun_round (double x);           /* 65 percent faster when rounding to an integer */
/* LUALIB_API double   sun_fmod (double x, double y);  */ /* 3 percent slower than GCC's implementation */
LUALIB_API double   sun_modf (double x, double *iptr);  /* 20 percent faster than GCC's implementation */
LUALIB_API double   sun_int (double x);
LUALIB_API double   sun_frac (double x);
LUALIB_API double   sun_hypot (double x, double y); /* a little bit slower than tools_hypot */
LUALIB_API double   sun_cbrt (double x);            /* 15 percent faster than GCC's implementation */
LUALIB_API double   sun_sin (double x);             /* 30 percent faster than GCC's implementation */
LUALIB_API double   sun_cos (double x);             /* 30 percent faster than GCC's implementation */
LUALIB_API double   sun_tan (double x);             /* eight percent faster than GCC's implementation */
LUALIB_API double   sun_asin (double x);            /* as fast as GCC's asin implementation */
LUALIB_API double   sun_acos (double x);            /* as fast as GCC's acos implementation */
LUALIB_API double   sun_atan (double x);            /* 45 percent faster than GCC's implementation */
LUALIB_API double   sun_atan2 (double y, double x); /* 20 percent faster than GCC's implementation */
LUALIB_API double   sun_sinh (double x);            /* 45 percent faster than GCC's implementation */
LUALIB_API double   sun_cosh (double x);            /* 45 percent faster than GCC's implementation */
/* LUALIB_API void     sun_sinhcosh (double x, double *sih, double *coh);  no benefit in speed with cosh(x) = sqrt(1+(sinh(x)**2) */
LUALIB_API double   sun_tanh (double x);            /* 55 percent faster than GCC's implementation */
LUALIB_API int      sun_rem_pio2 (double x, double *y);

LUALIB_API double   sun_asinh (double x);            /* 12 % faster than GCC's asinh */
/* LUALIB_API double   sun_acosh (double x); */      /* up to 65 % slower than GCC's acosh */
/* LUALIB_API double   sun_atanh (double x); */      /* half as fast as GCC's atanh */
LUALIB_API INLINE double   sun_pow (double x, double y);   /* 1.3 times faster than GCC's implementation, in some cases it is slower */

LUALIB_API double   sun_log (double x);             /* 20 percent faster than GCC's implementation */
LUALIB_API double   sun_log2 (double x);            /* 2.5 percent slower than GCC's log2 */
LUALIB_API double   tools_logbase (double x, double b);  /* 2.14.2 */
/* LUALIB_API double   sun_log1p (double x);           15 percent slower than GCC's log1p implementation */
LUALIB_API double   sun_exp (double x);             /* 60 percent faster than GCC's implementation */
LUALIB_API double   sun_expm1 (double x);           /* twice as fast as GCC's implementation */
LUALIB_API double   sun_log2 (double x);            /* seven percent slower than GCC's log2 implementation */
LUALIB_API double   sun_logb (double x);            /* as fast as GCC's logb */
LUALIB_API double   sun_log10 (double x);           /* four percent faster than GCC's log10 implementation */
LUALIB_API double   sun_ilogb (double x);           /* three percent faster than GCC's ilogb implementation */
LUALIB_API double   sun_fabs (double x);            /* as fast as GCC's fabs */
LUALIB_API double   sun_ldexp (double value, int exp);  /* 15 to 45 percent faster than GCC's ldexp */
#define sun_scalbn  sun_ldexp
LUALIB_API double   sun_frexp (double x, int *eptr);    /* 16 percent faster than GCC's ldexp */
/* LUALIB_API double   sun_erf (double x);             a little bit slower than GCC's erf implementation */
/* LUALIB_API double   sun_erfc (double x);            a third slower than than GCC's erfc implementation */
/* LUALIB_API double   sun_sqrt (double x);            DO NOT USE: seven times slower than GCC's sqrt implementation */
LUALIB_API double   sun_ceil (double x);            /* 14 percent faster than GCC's ceil implementation */
LUALIB_API double   sun_floor (double x);           /* 12 percent faster than GCC's floor implementation */
LUALIB_API double   ds_ldexp_exp (double x, int expt);
LUALIB_API double   ds_frexp_exp (double x, int *expt);
LUALIB_API int      tools_fpclassify (double x);
LUALIB_API double   tools_reducerange (double x, double min, double max);
LUALIB_API double   tools_branch (double x, int d);
LUALIB_API double   tools_matheps (double x);
LUALIB_API double   tools_binomial (double n, double k);
LUALIB_API double   tools_expx2 (double x, int sign);
LUALIB_API double   tools_cexpx2 (double a, double b, double sign, double *im);
LUALIB_API void     tools_sincos (double x, double *s, double *c);
LUALIB_API void     tools_sinhcosh (double x, double *sih, double *coh);

#ifndef PROPCMPLX
LUALIB_API double complex tools_csqrt (double complex z);
LUALIB_API double complex tools_cpow (double complex a, double complex z);  /* 13 percent faster than GCC's cpow implementation */
LUALIB_API double complex tools_cexp (double complex z);
LUALIB_API double complex tools_clog (double complex z);   /* 25 percent faster than GCC's clog */
LUALIB_API double complex tools_casin (double complex z);  /* twice as fast as the GCC implementation */
LUALIB_API double complex tools_csinh (double complex z);
LUALIB_API double complex tools_catan (double complex z);  /* 15 percent faster than GCC's catan */
LUALIB_API double complex tools_ctan (double complex z);   /* 25 percent faster than GCC's implementation */
LUALIB_API double complex tools_ctanh (double complex z);  /* 30 percent faster than GCC's ctanh */
LUALIB_API double complex ds_ldexp_cexp (double complex z, int expt);
LUALIB_API double tools_cabs (double complex z);
LUALIB_API double complex tools_cdiv (double complex x, double complex y);
LUALIB_API div_t tools_div (int n, int d);
#endif
LUALIB_API void     tools_sincosfast (float x, float *pS, float *pC);
LUALIB_API double   tools_sinfast (double x);
LUALIB_API double   tools_cosfast (double x);
LUALIB_API float    tools_sqrtfast (float x);
LUALIB_API float    tools_sqrtapx (float x);
LUALIB_API float    tools_lnfast (float x);
LUALIB_API unsigned int tools_modulo (unsigned int num, unsigned int div);  /* 2.12.0 RC 4 */
LUALIB_API int      tools_gcd (double x, double y);  /* 2.12.3 */
LUALIB_API double   tools_koadd (volatile double s, volatile double x, volatile double *q);  /* 2.13.0 */
LUALIB_API double   tools_polyeval (double x, double coef[], int n);  /* 2.13.0 */
LUALIB_API INLINE double tools_recip (double x);
LUALIB_API INLINE int      tools_isfloat (double x);
LUALIB_API INLINE int      tools_isint (double x);
LUALIB_API INLINE int      tools_isnegint (double x);
LUALIB_API INLINE int      tools_isposint (double x);
LUALIB_API INLINE int      tools_isnonnegint (double x);
LUALIB_API INLINE int      tools_isuint (double x, int bits);
LUALIB_API INLINE double   tools_random (int mode);
LUALIB_API INLINE int      tools_randomrange (int l, int u, int mode);
LUALIB_API INLINE int      tools_isevenorodd (double x);
#define tools_iseven(x)    (tools_isevenorodd(x) == 2)
#define tools_isodd(x)     (tools_isevenorodd(x) == 1)

#ifndef PROPCMPLX
LUALIB_API complex double tools_centier (agn_Complex a);
#else
LUALIB_API void     tools_centier (double a, double b, double *x, double *y);
#endif

/* setting and getting bits in an integer */

LUALIB_API INLINE void tools_setbit (int *x, int pos, int bit);
LUALIB_API INLINE int  tools_getbit (int x, int pos);
LUALIB_API INLINE uint32_t tools_onebits (register uint32_t x);
LUALIB_API INLINE uint32_t tools_flip (uint32_t n);
LUALIB_API INLINE int tools_remtrailzeros (register uint32_t n);
LUALIB_API INLINE uint32_t tools_clz32 (register uint32_t x, uint32_t *y);
LUALIB_API INLINE uint32_t tools_ctz32 (register uint32_t x, uint32_t *y);
LUALIB_API INLINE uint32_t tools_msb (register uint32_t x);
LUALIB_API INLINE uint32_t tools_lsb (register uint32_t x);
LUALIB_API INLINE uint32_t tools_fibmod (uint32_t k, uint32_t m, int exact);  /* 2.14.8 */

#define dsign(x)      ( ((x) == 0) ? 0 : (((x) < 0) ? -1 : 1) )

#define intdiv(a,b)   ( ((b) != 0) ? (dsign((a))*dsign((b))*sun_floor(fabs((a)/(b)))) : (AGN_NAN) )  /* more exact for neg values */

#define tools_isleapyear(y) ( ((y) % 4 == 0 && !((y) % 100 == 0)) || ((y) % 400 == 0) )

#define tools_exists(x) (access((x), 00|04) != -1)  /* 00 = file/dir exists, 04 = is readable */

LUALIB_API int tools_checkdatetime (int year, int month, int day, int hour, int minute, int second, int msecond);
LUALIB_API Time64_T tools_maketime (int year, int month, int day, int hour, int minute, int second, double *alternative);
LUALIB_API double tools_esd (int y, int m, int d, int hh, int mm, int ss);
LUALIB_API char *tools_dtoa (double x);
LUA_API    int tools_auxcdate (double x, int *iy, int *im, int *id, double *fd, int *deg, int *min, int *sec);

LUALIB_API void tools_commatodot (char *str, int *result);
LUALIB_API INLINE int tools_cpuid (uint32_t call, uint32_t *eax, uint32_t *ebx, uint32_t *ecx, uint32_t *edx);

#ifdef _WIN32
#define MS_WIN32S		1
#define MS_WIN95		2
#define MS_WINNT4		3
#define MS_WIN98		4
#define MS_WINME		5
#define MS_WIN2K		6
#define MS_WINXP		7
#define MS_WIN2003		8
#define MS_WINVISTA		9
#define MS_WINS2008		10
#define MS_WIN7			11
#define MS_WIN80		12
#define MS_WINS2012		13
#define MS_WIN81		14
#define MS_WINS2012R2	15
#define MS_WIN10		16
#define MS_WIN10S		17
#define MS_WIN10POST	18

typedef struct WinVer {
  int BuildNumber;
  int MajorVersion;
  int MinorVersion;
  int PlatformId;
  char ProductType;
} WinVer;

LUALIB_API int getWindowsVersion (struct WinVer *winver);
LUALIB_API int getSysOpType (int *minorVersion, int *BuildNumber, int *PlatformId, int *ProductType, double *winver, uint8_t *SPmaj, uint8_t *SPmin);
#endif

LUALIB_API char *getModuleFileName (void);

#if defined(_WIN32) || (defined(__SVR4) && defined(__sun))
int tools_vasprintf (char **sptr, char *fmt, va_list argv);
int tools_asprintf (char **sptr, char *fmt, ... );
#endif

LUALIB_API int tools_wait (double x);
LUALIB_API double tools_time (void);

/* Defines for tools_charmap */
#define __0____ 0x000
#define	__UPPER	0x001
#define	__LOWER	0x002
#define	__DIGIT	0x004
#define	__BLANK	0x008 /* HT  LF  VT  FF  CR  SP */
#define	__PUNCT	0x010
#define	__CNTRL	0x020
#define	__HEXAD	0x040
#define __PRINT 0x080

/* Defines for tools_alphadia */
#define	__ALPHA	0x001
#define __DIACR	0x002
#define __VOWEL 0x004
#define __UNKNO 0x008 /* not yet set or used */


/*
 * ====================================================
 * Copyright (C) 1993 by Sun Microsystems, Inc. All rights reserved.
 *
 * Developed at SunPro, a Sun Microsystems, Inc. business.
 * Permission to use, copy, modify, and distribute this
 * software is freely granted, provided that this notice
 * is preserved.
 * ====================================================
 */
/*
 * from: @(#)fdlibm.h 5.1 93/09/24
 * $FreeBSD: src/lib/msun/src/openlibm.h,v 1.82 2011/11/12 19:55:48 theraven Exp $
 */
/*
 * C99 specifies that complex numbers have the same representation as
 * an array of two elements, where the first element is the real part
 * and the second element is the imaginary part.
 */
#ifndef PROPCMPLX
typedef union {
  float complex f;
  float a[2];
} float_complex;

typedef union {
  double complex f;
  double a[2];
} double_complex;

typedef union {
  long double complex f;
  long double a[2];
} long_double_complex;
#define	REALPART(z)	((z).a[0])
#define	IMAGPART(z)	((z).a[1])

/*
 * Inline functions that can be used to construct complex values.
 *
 * The C99 standard intends x+I*y to be used for this, but x+I*y is
 * currently unusable in general since gcc introduces many overflow,
 * underflow, sign and efficiency bugs by rewriting I*y as
 * (0.0+I)*(y+0.0*I) and laboriously computing the full complex product.
 * In particular, I*Inf is corrupted to NaN+I*Inf, and I*-0 is corrupted
 * to -0.0+I*0.0.
 *
 * In C11, a CMPLX(x,y) macro was added to circumvent this limitation,
 * and gcc 4.7 added a __builtin_complex feature to simplify implementation
 * of CMPLX in libc, so we can take advantage of these features if they
 * are available.
 */
#if defined(CMPLXF) && defined(CMPLX) && defined(CMPLXL) /* C11 */
#  define cpackf(x,y) CMPLXF(x,y)
#  define cpack(x,y) CMPLX(x,y)
#  define cpackl(x,y) CMPLXL(x,y)
#elif (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 7)) && !defined(__INTEL_COMPILER)
#  define cpackf(x,y) __builtin_complex ((float) (x), (float) (y))
#  define cpack(x,y) __builtin_complex ((double) (x), (double) (y))
#  define cpackl(x,y) __builtin_complex ((long double) (x), (long double) (y))
#else /* define our own cpack functions */

/* Do NOT use INLINE macro as this causes trouble in OpenSUSE 10.3 and Mac OS X */
static __inline float complex cpackf(float x, float y) {
  float_complex z;
  REALPART(z) = x;
  IMAGPART(z) = y;
  return z.f;
}

static __inline double complex cpack (double x, double y) {
  double_complex z;
  REALPART(z) = x;
  IMAGPART(z) = y;
  return z.f;
}

static __inline long double complex cpackl (long double x, long double y) {
  long_double_complex z;
  REALPART(z) = x;
  IMAGPART(z) = y;
  return z.f;
}
#endif  /* define our own cpack functions */
#endif  /* of PROPCMPLX */
#endif

