#!/bin/sh
# compile and install skript for the plus package for Raspbian Stretch
# execute this batch file in the .../agena/src folder by typing:
# sh makepluslinux.sh
# -fPIC -mcpu=cortex-a53 -mfpu=neon-vfpv4  is a little bit slower
#export OPTIONS="-DLUA_USE_LINUX -DDEBIAN -Wall -O2 -fPIC -mcpu=cortex-a53 -mfpu=neon-vfpv4 -shared -I../src -L../src ../src/libagena.a"
export OPTIONS="-DLUA_USE_LINUX -DDEBIAN -Wall -O2 -shared -fgnu89-inline -I../src -L../src ../src/libagena.a"
#export MYFLAGS="-DDEBIAN"
export MYFLAGS=-fgnu89-inline
export EXPORTTO="../lib"

# delete *.o files not deleted by make clean
for i in ads.o calc.o linalg.o stats.o fractals.o phq.o xbase.o net.o interp.o moon.o sunriset.o astro.o skycrane.o llist.o bags.o iniparse.o hashes.o numarray.o factory.o bytes.o fastmath.o sema.o semalib.o dual.o utf8.o
do
   if [ -f i ]; then
      rm i
   fi
done

printf "Compiling ADS ... "
gcc -O2 $MYFLAGS -fpic -c -o ads.o ads.c
gcc $OPTIONS -fpic -o ads.so ads.o vecoff64.o
strip ads.so
mv -f ads.so $EXPORTTO
printf "done.\n"

if [ -f ../phq/phq.c ]; then
   printf "Compiling phonetiQs ... "
   gcc $OPTIONS -o phq.so ../phq/phq.c
   strip phq.so
   mv -f phq.so ../phq
   printf "done.\n"
fi

# do not use -O switch because this will lead to roundoffs in calc.fseq
printf "Compiling calc ... "
gcc -O2 $MYFLAGS -fpic -c -o calc.o calc.c
gcc -O2 $MYFLAGS -fpic -c -o interp.o interp.c
gcc -O2 $MYFLAGS -fpic -c -o rlhmath.o rlhmath.c
#gcc -O2 -shared -fpic -o calc.so calc.o interp.o rlhmath.o cephes.o -L. -lagena
gcc $OPTIONS -fpic -o calc.so calc.o interp.o rlhmath.o cephes.o
strip calc.so
mv -f calc.so $EXPORTTO
printf "done.\n"

printf "Compiling stats ... "
gcc -O2 $MYFLAGS -fpic -c -o stats.o stats.c
gcc $OPTIONS -fpic -o stats.so stats.o cephes.o
strip stats.so
mv -f stats.so $EXPORTTO
printf "done.\n"

printf "Compiling linalg ... "
gcc $OPTIONS -o linalg.so linalg.c
strip linalg.so
mv -f linalg.so $EXPORTTO
printf "done.\n"

printf "Compiling fractals ... "
gcc $OPTIONS -o fractals.so fractals.c
strip fractals.so
mv -f fractals.so $EXPORTTO
printf "done.\n"

printf "Compiling astro ... "
gcc -O2 $MYFLAGS -fpic -c -o astro.o astro.c
gcc -O2 $MYFLAGS -fpic -c -o sunriset.o sunriset.c
gcc -O2 $MYFLAGS -fpic -c -o moon.o moon.c
#gcc -O2 $MYFLAGS -shared -fpic -o astro.so astro.o sunriset.o moon.o sofa.o -L. -lagena
gcc $OPTIONS -fpic -o astro.so astro.o sunriset.o moon.o sofa.o
strip astro.so
mv -f astro.so $EXPORTTO
printf "done.\n"

printf "Compiling xbase ... "
gcc $OPTIONS -o xbase.so xbase.c
strip xbase.so
mv -f xbase.so $EXPORTTO
printf "done.\n"

printf "Compiling net ... "
gcc $OPTIONS -o net.so net.c
strip net.so
mv -f net.so $EXPORTTO
printf "done.\n"

printf "Compiling bags ... "
gcc $OPTIONS -o bags.so bags.c
strip bags.so
mv -f bags.so $EXPORTTO
printf "done.\n"

printf "Compiling skycrane ... "
gcc $OPTIONS -o skycrane.so skycrane.c
strip skycrane.so
mv -f skycrane.so $EXPORTTO
printf "done.\n"

printf "Compiling llist ... "
gcc $OPTIONS -o llist.so llist.c
strip llist.so
mv -f llist.so $EXPORTTO
printf "done.\n"

printf "Compiling hashes ... "
gcc $OPTIONS -o hashes.so hashes.c
strip hashes.so
mv -f hashes.so $EXPORTTO
printf "done.\n"

printf "Compiling numarray ... "
gcc $OPTIONS -o numarray.so numarray.c
strip numarray.so
mv -f numarray.so $EXPORTTO
printf "done.\n"

printf "Compiling zx ... "
gcc $OPTIONS -o zx.so zx.c
strip zx.so
mv -f zx.so $EXPORTTO
printf "done.\n"

printf "Compiling bloom ... "
gcc $OPTIONS -o bloom.so bloom.c
strip bloom.so
mv -f bloom.so $EXPORTTO
printf "done.\n"

printf "Compiling factory ... "
gcc $OPTIONS -o factory.so factory.c
strip factory.so
mv -f factory.so $EXPORTTO
printf "done.\n"

printf "Compiling fastmath ... "
gcc $OPTIONS -o fastmath.so fastmath.c
strip fastmath.so
mv -f fastmath.so $EXPORTTO
printf "done.\n"

printf "Compiling bytes ... "
gcc $OPTIONS -o bytes.so bytes.c
strip bytes.so
mv -f bytes.so $EXPORTTO
printf "done.\n"

printf "Compiling sema ... "

gcc -O2 $MYFLAGS -fpic -c -o semalib.o semalib.c
gcc -O2 $MYFLAGS -fpic -c -o sema.o sema.c
gcc $OPTIONS -fpic -o sema.so sema.o semalib.o
strip sema.so
mv -f sema.so $EXPORTTO
printf "done.\n"

printf "Compiling dual ... "
gcc $OPTIONS -o dual.so dual.c
strip dual.so
mv -f dual.so $EXPORTTO
printf "done.\n"

printf "Compiling utf8 ... "
gcc $OPTIONS -o utf8.so utf8.c
strip utf8.so
mv -f utf8.so $EXPORTTO
printf "done.\n"

printf "Compiling mp ... "
gcc $OPTIONS -o mp.so mp.c -lgmp
strip mp.so
mv -f mp.so $EXPORTTO
printf "done.\n"


#printf "Compiling compress ... "
#gcc $OPTIONS -o compress.so compress.c lcompress.c ../src/agnhlps.c
#strip compress.so
#mv -f compress.so $EXPORTTO
#printf "done.\n"

echo Installing all libraries into /lib folder ...
echo All done.


